#pragma once

namespace Qumela {

	using namespace Genode;

	class String_range_exception : public Exception
	{
	};
	
	template <typename T1>
	T1 string_part(const T1 &input, unsigned offset)
	{
		if (offset >= input.length()) {
			throw String_range_exception();
		}

		return T1(input.string() + offset);
	}

	template <typename T1>
	T1 string_part(const T1 &input, unsigned offset, unsigned length)
	{
		if (offset + length >= input.length()) {
			throw String_range_exception();
		}

		return T1(Cstring(input.string() + offset, length));
	}

	template <typename T1, typename T2>
	int string_find(const T1 &input, const T2 &find)
	{
		if (find.length() < 1) {
			throw String_range_exception();
		}

		if (find.length() > input.length()) {
			return -1;
		}

		for (unsigned i = 0; i <= input.length() - find.length(); i++) {
			if (string_part(input, i, find.length() - 1) == find) {
				return i;
			}
		}

		return -1;
	}

	template <typename T1, typename T2, typename T3>
	T1 string_replace(const T1 &input, const T2 &find, const T3 &replace)
	{
		if (find.length() < 1) {
			throw String_range_exception();
		}

		if (find.length() > input.length()) {
			return input;
		}

		T1 current = input;
		int i = string_find(current, find);

		while (i >= 0) {
			if (current.length() + replace.length() - find.length() > current.capacity()) {
				throw String_range_exception();
			}

			current = T1(string_part(current, 0, i), replace, string_part(current, i + find.length() - 1));
			i = string_find(current, find);
		}

		return current;
	}

	template <typename T>
	T string_split(const T &input, char delimiter, unsigned part)
	{
		int delimiter_index = -1;
		int last_delimiter = -1;
		char c[input.capacity()];
		memcpy(c, input.string(), input.capacity());

		for (unsigned i = 0; i < input.length(); i++)
		{
			if (c[i] == delimiter) {
				delimiter_index++;

				if (delimiter_index == (int)part)
				{
					c[i] = '\0';
					return T(Cstring(c + last_delimiter + 1));
				}

				last_delimiter = i;
			}
		}

		if (delimiter_index + 1 == (int)part) {
			return T(Cstring(c + last_delimiter + 1));
		} else {
			return T("");
		}
	}
	
	template <typename T>
	T path_split(const T &input, unsigned part)
	{
		return string_split(input, '/', part);
	}

	template <typename T>
	bool is_greater(const T &a, const T &b)
	{
		const char* as = a.string();
		const char* bs = b.string();
		unsigned length = min(a.length(), b.length());

		for (unsigned i = 0; i < length; i++) {
			if (as[i] > bs[i]) {
				return true;
			} else if (as[i] < bs[i]) {
				return false;
			}
		}

		if (a.length() > b.length()) {
			return true;
		} else {
			return false;
		}
	}
}
